@@ -1,6 +1,6 @@
 /* -*- mode:C; c-file-style: "bsd" -*- */
 /*
- * Copyright (c) 2009-2012 Yubico AB
+ * Copyright (c) 2009-2013 Yubico AB
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -29,6 +29,7 @@
  */
 
 #include <ykpers.h>
+#include <ykpers-version.h>
 #include <stdio.h>
 
 int main (void)
@@ -36,6 +37,25 @@ int main (void)
 	YKP_CONFIG *ykp;
 	int rc;
 
+	if (strcmp (YKPERS_VERSION_STRING, ykpers_check_version (NULL)) != 0)
+	{
+		printf ("version mismatch %s != %s\n",YKPERS_VERSION_STRING,
+			ykpers_check_version (NULL));
+		return 1;
+	}
+
+	if (ykpers_check_version (YKPERS_VERSION_STRING) == NULL)
+	{
+		printf ("version NULL?\n");
+		return 1;
+	}
+
+	if (ykpers_check_version ("99.99.99") != NULL)
+	{
+		printf ("version not NULL?\n");
+		return 1;
+	}
+
 	ykp = ykp_alloc ();
 	if (!ykp)
 	{
