@@ -1,6 +1,6 @@
 /* -*- mode:C; c-file-style: "bsd" -*- */
 /*
- * Copyright (c) 2008-2012 Yubico AB
+ * Copyright (c) 2008-2013 Yubico AB
  * Copyright (c) 2010 Tollef Fog Heen <tfheen@err.no>
  * All rights reserved.
  *
@@ -41,32 +41,29 @@
 
 #include "ykpers-args.h"
 
-static int reader(char *buf, size_t count, void *stream)
-{
-	return (int)fread(buf, 1, count, (FILE *)stream);
-}
-static int writer(const char *buf, size_t count, void *stream)
-{
-	return (int)fwrite(buf, 1, count, (FILE *)stream);
-}
-
 int main(int argc, char **argv)
 {
 	FILE *inf = NULL; const char *infname = NULL;
 	FILE *outf = NULL; const char *outfname = NULL;
+	int data_format = YKP_FORMAT_LEGACY;
 	bool verbose = false;
 	bool aesviahash = false;
 	bool use_access_code = false;
 	unsigned char access_code[256];
+	unsigned char scan_codes[sizeof(SCAN_MAP)];
 	YK_KEY *yk = 0;
 	YKP_CONFIG *cfg = ykp_alloc();
 	YK_STATUS *st = ykds_alloc();
 	bool autocommit = false;
+	char data[1024];
+	bool dry_run = false;
 
 	/* Options */
 	char *salt = NULL;
 	char ndef_string[128] = {0};
-	char ndef_type = NULL;
+	char ndef_type = 0;
+	unsigned char usb_mode = 0;
+	bool zap = false;
 
 	bool error = false;
 	int exit_code = 0;
@@ -108,7 +105,7 @@ int main(int argc, char **argv)
 	else
 		printf("Unconfigured\n");
 
-	if (!(yk_check_firmware_version(yk))) {
+	if (!(yk_check_firmware_version2(st))) {
 		if (yk_errno == YK_EFIRMWARE) {
 			printf("Unsupported firmware revision - some "
 			       "features may not be available\n"
@@ -123,11 +120,12 @@ int main(int argc, char **argv)
 	/* Parse all arguments in a testable way */
 	if (! args_to_config(argc, argv, cfg, yk,
 			     &infname, &outfname,
+			     &data_format,
 			     &autocommit, salt,
-			     st, &verbose,
+			     st, &verbose, &dry_run,
 			     access_code, &use_access_code,
 			     &aesviahash, &ndef_type, ndef_string,
-			     &exit_code)) {
+			     &usb_mode, &zap, scan_codes, &exit_code)) {
 		goto err;
 	}
 
@@ -178,13 +176,22 @@ int main(int argc, char **argv)
 	}
 
 	if (inf) {
-		if (!ykp_read_config(cfg, reader, inf))
+		if(!ykp_clear_config(cfg))
 			goto err;
-	} else if (! aesviahash && (ykp_command(cfg) == SLOT_CONFIG || ykp_command(cfg) == SLOT_CONFIG2)) {
+		if(!fread(data, 1, 1024, inf))
+			goto err;
+		if (!ykp_import_config(cfg, data, strlen(data), data_format))
+			goto err;
+	} else if (! aesviahash && ! zap && (ykp_command(cfg) == SLOT_CONFIG || ykp_command(cfg) == SLOT_CONFIG2)) {
 		char passphrasebuf[256]; size_t passphraselen;
 		fprintf(stderr, "Passphrase to create AES key: ");
 		fflush(stderr);
-		fgets(passphrasebuf, sizeof(passphrasebuf), stdin);
+		if (!fgets(passphrasebuf, sizeof(passphrasebuf), stdin))
+		{
+			perror ("fgets");
+			exit_code = 1;
+			goto err;
+		}
 		passphraselen = strlen(passphrasebuf);
 		if (passphrasebuf[passphraselen - 1] == '\n')
 			passphrasebuf[passphraselen - 1] = '\0';
@@ -194,29 +201,44 @@ int main(int argc, char **argv)
 	}
 
 	if (outf) {
-		if (!ykp_write_config(cfg, writer, outf))
+		if(!(ykp_export_config(cfg, data, 1024, data_format))) {
 			goto err;
+		}
+		if(!(fwrite(data, 1, strlen(data), outf))) {
+			goto err;
+		}
 	} else {
 		char commitbuf[256]; size_t commitlen;
 
 		if (ykp_command(cfg) == SLOT_SWAP) {
 			fprintf(stderr, "Configuration in slot 1 and 2 will be swapped\n");
-		} else if(ykp_command(cfg) == SLOT_NDEF) {
-			fprintf(stderr, "New NDEF URI will be written\n");
+		} else if(ykp_command(cfg) == SLOT_NDEF || ykp_command(cfg) == SLOT_NDEF2) {
+			fprintf(stderr, "New NDEF will be written as:\n%s\n", ndef_string);
+		} else if(ykp_command(cfg) == SLOT_DEVICE_CONFIG) {
+			fprintf(stderr, "The USB mode will be set to: 0x%x\n", usb_mode);
+		} else if(ykp_command(cfg) == SLOT_SCAN_MAP) {
+			fprintf(stderr, "A new scanmap will be written.\n");
+		} else if(zap) {
+			fprintf(stderr, "Configuration in slot %d will be deleted\n", ykp_config_num(cfg));
 		} else {
 			if (ykp_command(cfg) == SLOT_CONFIG || ykp_command(cfg) == SLOT_CONFIG2) {
 				fprintf(stderr, "Configuration data to be written to key configuration %d:\n\n", ykp_config_num(cfg));
 			} else {
 				fprintf(stderr, "Configuration data to be updated in key configuration %d:\n\n", ykp_command(cfg) == SLOT_UPDATE1 ? 1 : 2);
 			}
-			ykp_write_config(cfg, writer, stderr);
+			ykp_export_config(cfg, data, 1024, YKP_FORMAT_LEGACY);
+			fwrite(data, 1, strlen(data), stderr);
 		}
 		fprintf(stderr, "\nCommit? (y/n) [n]: ");
 		if (autocommit) {
 			strcpy(commitbuf, "yes");
 			puts(commitbuf);
 		} else {
-			fgets(commitbuf, sizeof(commitbuf), stdin);
+			if (!fgets(commitbuf, sizeof(commitbuf), stdin))
+			{
+				perror ("fgets");
+				goto err;
+			}
 		}
 		commitlen = strlen(commitbuf);
 		if (commitbuf[commitlen - 1] == '\n')
@@ -227,25 +249,54 @@ int main(int argc, char **argv)
 
 			if (verbose)
 				printf("Attempting to write configuration to the yubikey...");
-			if(ykp_command(cfg) == SLOT_NDEF) {
-				YKNDEF ndef;
-				memset(&ndef, 0, sizeof(YKNDEF));
+			if (dry_run) {
+				printf("Not writing anything to key due to dry_run requested.\n");
+			}
+			else if(ykp_command(cfg) == SLOT_NDEF || ykp_command(cfg) == SLOT_NDEF2) {
+				YK_NDEF *ndef = ykp_alloc_ndef();
+				int confnum = 1;
 				if(ndef_type == 'U') {
-					ykp_construct_ndef_uri(&ndef, ndef_string);
+					ykp_construct_ndef_uri(ndef, ndef_string);
 				} else if(ndef_type == 'T') {
-					ykp_construct_ndef_text(&ndef, ndef_string, "en", false);
+					ykp_construct_ndef_text(ndef, ndef_string, "en", false);
 				}
 				if(use_access_code) {
-					memcpy(ndef.curAccCode, access_code, ACC_CODE_SIZE);
+					ykp_set_ndef_access_code(ndef, access_code);
 				}
-				if (!yk_write_ndef(yk, &ndef)) {
+				if(ykp_command(cfg) == SLOT_NDEF2) {
+					confnum = 2;
+				}
+				if (!yk_write_ndef2(yk, ndef, confnum)) {
 					if (verbose)
 						printf(" failure\n");
 					goto err;
 				}
+				ykp_free_ndef(ndef);
+			} else if(ykp_command(cfg) == SLOT_DEVICE_CONFIG) {
+				YK_DEVICE_CONFIG *device_config = ykp_alloc_device_config();
+				ykp_set_device_mode(device_config, usb_mode);
+				if(!yk_write_device_config(yk, device_config)) {
+					if(verbose)
+						printf(" failure\n");
+					goto err;
+				}
+				ykp_free_device_config(device_config);
+
+
+			} else if(ykp_command(cfg) == SLOT_SCAN_MAP) {
+				if(!yk_write_scan_map(yk, scan_codes)) {
+					if(verbose)
+						printf(" failure\n");
+					goto err;
+				}
 			} else {
+				YK_CONFIG *ycfg = NULL;
+				/* if we're deleting a slot we send the configuration as NULL */
+				if (!zap) {
+					ycfg = ykp_core_config(cfg);
+				}
 				if (!yk_write_command(yk,
-							ykp_core_config(cfg), ykp_command(cfg),
+							ycfg, ykp_command(cfg),
 							use_access_code ? access_code : NULL)) {
 					if (verbose)
 						printf(" failure\n");
@@ -253,7 +304,7 @@ int main(int argc, char **argv)
 				}
 			}
 
-			if (verbose)
+			if (verbose && !dry_run)
 				printf(" success\n");
 		}
 	}
